#define PORT    3125
#define WRITE_COMBINING_WINDOW 1400
#define MAX_BYTES_TO_QUEUE (WRITE_COMBINING_WINDOW*3)
// Close client if it misses this many consecutive writes.
#define MAX_NETWORK_WRITES_TO_SKIP 20

typedef enum {
  DISCARD_IF_BLOCK,
  QUEUE_IF_BLOCK,
  BLOCK_IF_BLOCK
} network_blocking_behavior;

typedef struct network_client_output_fifo {
  struct network_client_output_fifo *next;
  char *buffer;
  int start;			/* initially 0, increments */
  int len;			/* initially len of buffer, decrements */
} network_client_output_fifo;

typedef enum {
  NEW, OPEN, CLOSING
} network_client_state;

typedef struct network_client {
  GMutex *mutex;
  struct network_client *next;
  int tx_authorized;
  int tx;
  int state;

  struct sockaddr_in *client_sockaddr_in;
  int fd;

  int waterfall_network_enable;
  int waterfall_skip_amount;
  int waterfall_skip_index;
  int waterfall_enable_frequency;
  int waterfall_last_frequency;
  int waterfall_width;
  int waterfall_last_bandwidth;

  int miniscope_network_enable;
  int miniscope_skip_amount;
  int miniscope_skip_index;

  int network_writes_skipped;

  char *last_mode;

  network_client_output_fifo *fifo;

} network_client;

void network_update_waterfall_frequency(int x2);
void network_update_waterfall_bandwidth(gdouble bw);
void network_update_mode(char *modeName);
void network_send_tx_char(gchar c);
void network_send_rx_char(gchar c);

// Network low levels
extern void network_init(gchar *password);
extern void network_process();
extern int network_poll(char *buffer, int len, network_client **ncptr);
extern int network_client_write (network_client *nc, char *buffer, int len, network_blocking_behavior ifBlock);
extern int network_read_from_client (network_client *nc, char *buffer, int len);

// Gunk
extern int network_clients_n;
extern network_client *network_clients;

